<?php
// This file is part of the local fullscreen plugin for Moodle
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * File containing code that requires the javascript for the fullscreen toggle button
 *
 * @package    local_fullscreen
 * @copyright  2014 onwards - University of Nottingham <www.nottingham.ac.uk>
 * @author     Barry Oosthuizen <barry.oosthuizen@nottingham.ac.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Tests if the fullscreen button should be added to the page.
 *
 * @return bool
 */
function local_fullscreen_add_to_page(): bool {
    global $PAGE;

    // Page layouts that should not have the fullscreen button.
    $pagelayout = [
        'login',
        'embedded',
        'popup',
        'redirect',
        'frametop',
        'maintenance',
        'mydashboard',
        'mycourses',
    ];
    $excludelayout = in_array($PAGE->pagelayout, $pagelayout, true);

    // Page types that should not have the fullscreen button.
    $pagetype = [
        'mod-quiz-attempt',
        'mod-quiz-review',
        'mod-quiz-summary',
    ];
    $excludetype = in_array($PAGE->pagetype, $pagetype, true);

    if (CLI_SCRIPT || AJAX_SCRIPT || $excludelayout || $excludetype) {
        return false;
    }
    return true;
}

/**
 * Adds some extra classes to help identify the mode that the fullscreen button should work in.
 *
 * @see https://docs.moodle.org/dev/Output_callbacks#before_http_headers
 *
 * @return void
 */
function local_fullscreen_before_http_headers() {
    global $PAGE;
    if (!local_fullscreen_add_to_page()) {
        return;
    }

    $class = 'local-fullscreen-boost';

    if ($PAGE->theme->name === 'classic' || in_array('classic', $PAGE->theme->parents)) {
        $class = 'local-fullscreen-classic';
    }

    $PAGE->add_body_class($class);
}

/**
 * Adds the full screen button to the page.
 *
 * @see https://docs.moodle.org/dev/Output_callbacks#before_footer
 *
 * @return void
 */
function local_fullscreen_before_footer() {
    global $PAGE;
    if (!local_fullscreen_add_to_page()) {
        return;
    }
    $params = [
        'fullscreen' => get_user_preferences('fullscreenmode', false),
        'toggle' => get_config('local_fullscreen', 'toggle', 'Ctrl + Alt + b'),
    ];
    $PAGE->requires->js_call_amd('local_fullscreen/button', 'init', $params);
}

/**
 * Returns the name of the user preferences as well as the details this plugin uses.
 *
 * @return array
 */
function local_fullscreen_user_preferences() {
    $preferences = array();
    $preferences['fullscreenmode'] = array(
        'type' => PARAM_BOOL,
        'null' => NULL_NOT_ALLOWED,
        'default' => false,
    );

    return $preferences;
}
